@@ -1,25 +1,34 @@
-from .base import (
-    Submodule,
-    UpdateProgress
-)
-from .util import (
-    find_first_remote_branch
-)
+from .base import Submodule, UpdateProgress
+from .util import find_first_remote_branch
 from git.exc import InvalidGitRepositoryError
 import git
 
 import logging
 
+# typing -------------------------------------------------------------------
+
+from typing import TYPE_CHECKING, Union
+
+from git.types import Commit_ish
+
+if TYPE_CHECKING:
+    from git.repo import Repo
+    from git.util import IterableList
+
+# ----------------------------------------------------------------------------
+
 __all__ = ["RootModule", "RootUpdateProgress"]
 
-log = logging.getLogger('git.objects.submodule.root')
+log = logging.getLogger("git.objects.submodule.root")
 log.addHandler(logging.NullHandler())
 
 
 class RootUpdateProgress(UpdateProgress):
     """Utility class which adds more opcodes to the UpdateProgress"""
+
     REMOVE, PATHCHANGE, BRANCHCHANGE, URLCHANGE = [
-        1 << x for x in range(UpdateProgress._num_op_codes, UpdateProgress._num_op_codes + 4)]
+        1 << x for x in range(UpdateProgress._num_op_codes, UpdateProgress._num_op_codes + 4)
+    ]
     _num_op_codes = UpdateProgress._num_op_codes + 4
 
     __slots__ = ()
@@ -40,30 +49,39 @@ class RootModule(Submodule):
 
     __slots__ = ()
 
-    k_root_name = '__ROOT__'
+    k_root_name = "__ROOT__"
 
-    def __init__(self, repo):
+    def __init__(self, repo: "Repo"):
         # repo, binsha, mode=None, path=None, name = None, parent_commit=None, url=None, ref=None)
         super(RootModule, self).__init__(
             repo,
             binsha=self.NULL_BIN_SHA,
             mode=self.k_default_mode,
-            path='',
+            path="",
             name=self.k_root_name,
             parent_commit=repo.head.commit,
-            url='',
-            branch_path=git.Head.to_full_path(self.k_head_default)
+            url="",
+            branch_path=git.Head.to_full_path(self.k_head_default),
         )
 
-    def _clear_cache(self):
+    def _clear_cache(self) -> None:
         """May not do anything"""
         pass
 
-    #{ Interface
+    # { Interface
 
-    def update(self, previous_commit=None, recursive=True, force_remove=False, init=True,
-               to_latest_revision=False, progress=None, dry_run=False, force_reset=False,
-               keep_going=False):
+    def update(
+        self,
+        previous_commit: Union[Commit_ish, None] = None,  # type: ignore[override]
+        recursive: bool = True,
+        force_remove: bool = False,
+        init: bool = True,
+        to_latest_revision: bool = False,
+        progress: Union[None, "RootUpdateProgress"] = None,
+        dry_run: bool = False,
+        force_reset: bool = False,
+        keep_going: bool = False,
+    ) -> "RootModule":
         """Update the submodules of this repository to the current HEAD commit.
         This method behaves smartly by determining changes of the path of a submodules
         repository, next to changes to the to-be-checked-out commit or the branch to be
@@ -104,9 +122,9 @@ class RootModule(Submodule):
             progress = RootUpdateProgress()
         # END assure progress is set
 
-        prefix = ''
+        prefix = ""
         if dry_run:
-            prefix = 'DRY-RUN: '
+            prefix = "DRY-RUN: "
 
         repo = self.repo
 
@@ -125,17 +143,17 @@ class RootModule(Submodule):
                     previous_commit = cur_commit
                 # END exception handling
             else:
-                previous_commit = repo.commit(previous_commit)   # obtain commit object
+                previous_commit = repo.commit(previous_commit)  # obtain commit object
             # END handle previous commit
 
-            psms = self.list_items(repo, parent_commit=previous_commit)
-            sms = self.list_items(repo)
+            psms: "IterableList[Submodule]" = self.list_items(repo, parent_commit=previous_commit)
+            sms: "IterableList[Submodule]" = self.list_items(repo)
             spsms = set(psms)
             ssms = set(sms)
 
             # HANDLE REMOVALS
             ###################
-            rrsm = (spsms - ssms)
+            rrsm = spsms - ssms
             len_rrsm = len(rrsm)
 
             for i, rsm in enumerate(rrsm):
@@ -146,9 +164,19 @@ class RootModule(Submodule):
 
                 # fake it into thinking its at the current commit to allow deletion
                 # of previous module. Trigger the cache to be updated before that
-                progress.update(op, i, len_rrsm, prefix + "Removing submodule %r at %s" % (rsm.name, rsm.abspath))
+                progress.update(
+                    op,
+                    i,
+                    len_rrsm,
+                    prefix + "Removing submodule %r at %s" % (rsm.name, rsm.abspath),
+                )
                 rsm._parent_commit = repo.head.commit
-                rsm.remove(configuration=False, module=True, force=force_remove, dry_run=dry_run)
+                rsm.remove(
+                    configuration=False,
+                    module=True,
+                    force=force_remove,
+                    dry_run=dry_run,
+                )
 
                 if i == len_rrsm - 1:
                     op |= END
@@ -159,24 +187,31 @@ class RootModule(Submodule):
             # HANDLE PATH RENAMES
             #####################
             # url changes + branch changes
-            csms = (spsms & ssms)
+            csms = spsms & ssms
             len_csms = len(csms)
             for i, csm in enumerate(csms):
-                psm = psms[csm.name]
-                sm = sms[csm.name]
+                psm: "Submodule" = psms[csm.name]
+                sm: "Submodule" = sms[csm.name]
 
                 # PATH CHANGES
                 ##############
                 if sm.path != psm.path and psm.module_exists():
-                    progress.update(BEGIN | PATHCHANGE, i, len_csms, prefix +
-                                    "Moving repository of submodule %r from %s to %s"
-                                    % (sm.name, psm.abspath, sm.abspath))
+                    progress.update(
+                        BEGIN | PATHCHANGE,
+                        i,
+                        len_csms,
+                        prefix + "Moving repository of submodule %r from %s to %s" % (sm.name, psm.abspath, sm.abspath),
+                    )
                     # move the module to the new path
                     if not dry_run:
                         psm.move(sm.path, module=True, configuration=False)
                     # END handle dry_run
                     progress.update(
-                        END | PATHCHANGE, i, len_csms, prefix + "Done moving repository of submodule %r" % sm.name)
+                        END | PATHCHANGE,
+                        i,
+                        len_csms,
+                        prefix + "Done moving repository of submodule %r" % sm.name,
+                    )
                 # END handle path changes
 
                 if sm.module_exists():
@@ -186,14 +221,18 @@ class RootModule(Submodule):
                         # Add the new remote, remove the old one
                         # This way, if the url just changes, the commits will not
                         # have to be re-retrieved
-                        nn = '__new_origin__'
+                        nn = "__new_origin__"
                         smm = sm.module()
                         rmts = smm.remotes
 
                         # don't do anything if we already have the url we search in place
                         if len([r for r in rmts if r.url == sm.url]) == 0:
-                            progress.update(BEGIN | URLCHANGE, i, len_csms, prefix +
-                                            "Changing url of submodule %r from %s to %s" % (sm.name, psm.url, sm.url))
+                            progress.update(
+                                BEGIN | URLCHANGE,
+                                i,
+                                len_csms,
+                                prefix + "Changing url of submodule %r from %s to %s" % (sm.name, psm.url, sm.url),
+                            )
 
                             if not dry_run:
                                 assert nn not in [r.name for r in rmts]
@@ -230,7 +269,8 @@ class RootModule(Submodule):
                                         # Alternatively we could just generate a unique name and leave all
                                         # existing ones in place
                                         raise InvalidGitRepositoryError(
-                                            "Couldn't find original remote-repo at url %r" % psm.url)
+                                            "Couldn't find original remote-repo at url %r" % psm.url
+                                        )
                                     # END handle one single remote
                                 # END handle check we found a remote
 
@@ -265,15 +305,22 @@ class RootModule(Submodule):
                                     # this way, it will be checked out in the next step
                                     # This will change the submodule relative to us, so
                                     # the user will be able to commit the change easily
-                                    log.warning("Current sha %s was not contained in the tracking\
-             branch at the new remote, setting it the the remote's tracking branch", sm.hexsha)
+                                    log.warning(
+                                        "Current sha %s was not contained in the tracking\
+             branch at the new remote, setting it the the remote's tracking branch",
+                                        sm.hexsha,
+                                    )
                                     sm.binsha = rref.commit.binsha
                                 # END reset binsha
 
                                 # NOTE: All checkout is performed by the base implementation of update
                             # END handle dry_run
                             progress.update(
-                                END | URLCHANGE, i, len_csms, prefix + "Done adjusting url of submodule %r" % (sm.name))
+                                END | URLCHANGE,
+                                i,
+                                len_csms,
+                                prefix + "Done adjusting url of submodule %r" % (sm.name),
+                            )
                         # END skip remote handling if new url already exists in module
                     # END handle url
 
@@ -282,9 +329,14 @@ class RootModule(Submodule):
                     if sm.branch_path != psm.branch_path:
                         # finally, create a new tracking branch which tracks the
                         # new remote branch
-                        progress.update(BEGIN | BRANCHCHANGE, i, len_csms, prefix +
-                                        "Changing branch of submodule %r from %s to %s"
-                                        % (sm.name, psm.branch_path, sm.branch_path))
+                        progress.update(
+                            BEGIN | BRANCHCHANGE,
+                            i,
+                            len_csms,
+                            prefix
+                            + "Changing branch of submodule %r from %s to %s"
+                            % (sm.name, psm.branch_path, sm.branch_path),
+                        )
                         if not dry_run:
                             smm = sm.module()
                             smmr = smm.remotes
@@ -294,7 +346,11 @@ class RootModule(Submodule):
                             # end for each remote
 
                             try:
-                                tbr = git.Head.create(smm, sm.branch_name, logmsg='branch: Created from HEAD')
+                                tbr = git.Head.create(
+                                    smm,
+                                    sm.branch_name,
+                                    logmsg="branch: Created from HEAD",
+                                )
                             except OSError:
                                 # ... or reuse the existing one
                                 tbr = git.Head(smm, sm.branch_path)
@@ -309,7 +365,11 @@ class RootModule(Submodule):
                         # END handle dry_run
 
                         progress.update(
-                            END | BRANCHCHANGE, i, len_csms, prefix + "Done changing branch of submodule %r" % sm.name)
+                            END | BRANCHCHANGE,
+                            i,
+                            len_csms,
+                            prefix + "Done changing branch of submodule %r" % sm.name,
+                        )
                     # END handle branch
                 # END handle
             # END for each common submodule
@@ -323,28 +383,44 @@ class RootModule(Submodule):
         ######################################
         for sm in sms:
             # update the submodule using the default method
-            sm.update(recursive=False, init=init, to_latest_revision=to_latest_revision,
-                      progress=progress, dry_run=dry_run, force=force_reset, keep_going=keep_going)
+            sm.update(
+                recursive=False,
+                init=init,
+                to_latest_revision=to_latest_revision,
+                progress=progress,
+                dry_run=dry_run,
+                force=force_reset,
+                keep_going=keep_going,
+            )
 
-            # update recursively depth first - question is which inconsitent
+            # update recursively depth first - question is which inconsistent
             # state will be better in case it fails somewhere. Defective branch
             # or defective depth. The RootSubmodule type will never process itself,
             # which was done in the previous expression
             if recursive:
                 # the module would exist by now if we are not in dry_run mode
                 if sm.module_exists():
-                    type(self)(sm.module()).update(recursive=True, force_remove=force_remove,
-                                                   init=init, to_latest_revision=to_latest_revision,
-                                                   progress=progress, dry_run=dry_run, force_reset=force_reset,
-                                                   keep_going=keep_going)
+                    type(self)(sm.module()).update(
+                        recursive=True,
+                        force_remove=force_remove,
+                        init=init,
+                        to_latest_revision=to_latest_revision,
+                        progress=progress,
+                        dry_run=dry_run,
+                        force_reset=force_reset,
+                        keep_going=keep_going,
+                    )
                 # END handle dry_run
             # END handle recursive
         # END for each submodule to update
 
         return self
 
-    def module(self):
+    def module(self) -> "Repo":
         """:return: the actual repository containing the submodules"""
         return self.repo
-    #} END interface
-#} END classes
+
+    # } END interface
+
+
+# } END classes
